// @flow
import _ from 'lodash';
import base from './base';
import {compose, lifecycle, withProps, mapProps} from 'recompose';
import {default as rest} from '../rest';

export default (hocParams: {
	baseURL?: string,
	entity: string | (props: Object) => string,
	key: string | (props: Object) => string,
	requests?: (props: Object) => [],
}) => (WrappedComponent: any) => compose(
	base(hocParams),

	mapProps(props => ({
		...props,

		faFetched: {
			...props.faFetched,

			[props.resolvedKey]: {
				...props.faFetched[props.resolvedKey],
				requests: hocParams.requests && _.isFunction(hocParams.requests) ? hocParams.requests(props) : [],

				post: (requests: [], callback?: Function) => props.dispatch(
					rest({
						baseURL: props.faFetched[props.resolvedKey].baseURL,
						entity : props.resolvedEntity,
						key    : props.resolvedKey,
					}).actions[props.resolvedKey].post(
						null,
						{body: JSON.stringify(requests)},
						callback
					)
				),
			},
		},
	})),

	lifecycle({
		componentDidMount() {
			if(hocParams.fetchOnMount && this.props.faFetched[this.props.resolvedKey].requests.length)
				this.props.faFetched[this.props.resolvedKey].post(this.props.faFetched[this.props.resolvedKey].requests);
		},

		componentWillReceiveProps(nextProps) {
			if(nextProps.hasPropsChanged({nextProps, props: this.props, trackProps: ['requests']}))
				nextProps.faFetched[nextProps.resolvedKey].post(nextProps.faFetched[nextProps.resolvedKey].requests);
		},
	}),
)(WrappedComponent);
